@@ -1,7 +1,11 @@
 from typing import List
+import dateutil.parser
+from datetime import datetime, timedelta
 
 from . import htb
 from .solve import MachineSolve
+from .errors import IncorrectArgumentException, IncorrectFlagException
+from .utils import parse_delta
 
 
 class Machine(htb.HTBObject):
@@ -24,7 +28,7 @@ class Machine(htb.HTBObject):
 
         active: Whether the Machine is active
         retired: Whether the Machine is retired
-        difficulty_number: The numeric difficulty of the Machine
+        avg_difficulty: The average numeric difficulty of the Machine
         completed: Whether the active User has completed the Machine
          :noindex: user_own_time: How long the active User took to own user
          :noindex: root_own_time: How long the active User took to own root
@@ -32,11 +36,12 @@ class Machine(htb.HTBObject):
         root_blood: The Solve of the Machine's first root blood
         user_own_time: How long the first User took to own user
         root_own_time: How long the first User took to own root
+        difficulty_ratings: A dict of difficulty ratings given
     """
     name: str = None
     os: str = None
     points: int = None
-    release_date: str = None
+    release_date: datetime = None
     user_owns: int = None
     root_owns: int = None
     free: bool = None
@@ -48,22 +53,43 @@ class Machine(htb.HTBObject):
     difficulty: str = None
 
     _detailed_attributes = ('active', 'retired', 'user_own_time', 'root_own_time', 'user_blood',
-                            'root_blood', 'user_blood_time', 'root_blood_time')
+                            'root_blood', 'user_blood_time', 'root_blood_time', 'difficulty_ratings')
     active: bool = None
     retired: bool = None
-    difficulty_number: int = None
+    avg_difficulty: int = None
     completed: bool = None
-    user_own_time: str = None
-    root_own_time: str = None
+    user_own_time: timedelta = None
+    root_own_time: timedelta = None
     user_blood: MachineSolve = None
     root_blood: MachineSolve = None
-    user_blood_time: str = None
-    root_blood_time: str = None
+    user_blood_time: timedelta = None
+    root_blood_time: timedelta = None
+    difficulty_ratings: dict = None
 
     # noinspection PyUnresolvedReferences
     _authors: List["User"] = None
     _author_ids: List[int] = None
 
+    def submit(self, flag: str, difficulty: int):
+        """ Submits a flag for a Machine
+
+        Args:
+            flag: The flag for the Machine
+            difficulty: A rating between 10 and 100 of the Machine difficulty
+
+        """
+        if difficulty < 10 or difficulty > 100 or difficulty % 10 != 0:
+            raise IncorrectArgumentException
+
+        submission = self._client.do_request("machine/own", json_data={
+            "flag": flag,
+            "id": self.id,
+            "difficulty": difficulty
+        })
+        if submission['message'] == "Incorrect flag!":
+            raise IncorrectFlagException
+        return True
+
     # noinspection PyUnresolvedReferences
     @property
     def authors(self) -> List["User"]:
@@ -88,7 +114,7 @@ class Machine(htb.HTBObject):
         self.name = data['name']
         self.os = data['os']
         self.points = data['points']
-        self.release_date = data['release']
+        self.release_date = dateutil.parser.parse(data['release'])
         self.user_owns = data['user_owns_count']
         self.root_owns = data['root_owns_count']
         self.user_owned = data['authUserInUserOwns']
@@ -104,25 +130,28 @@ class Machine(htb.HTBObject):
         if not summary:
             self.active = bool(data['active'])
             self.retired = bool(data['retired'])
-            self.user_own_time = data['authUserFirstUserTime']
-            self.root_own_time = data['authUserFirstRootTime']
+            self.user_own_time = parse_delta(data['authUserFirstUserTime'])
+            self.root_own_time = parse_delta(data['authUserFirstRootTime'])
+            self.difficulty_ratings = data['feedbackForChart']
             if data['userBlood']:
                 user_blood_data = {
-                    "date": data['userBlood']['created_at'],
+                    "date": dateutil.parser.parse(data['userBlood']['created_at']),
                     "first_blood": True,
                     "id": data['id'],
                     "name": data['name'],
                     "type": "user"
                 }
                 self.user_blood = MachineSolve(user_blood_data, self._client)
-                self.user_blood_time = data['userBlood']['blood_difference']
+                self.user_blood_time = parse_delta(data['userBlood']['blood_difference'])
             if data['rootBlood']:
                 user_blood_data = {
-                    "date": data['rootBlood']['created_at'],
+                    "date": dateutil.parser.parse(data['rootBlood']['created_at']),
                     "first_blood": True,
                     "id": data['id'],
                     "name": data['name'],
                     "type": "root"
                 }
                 self.root_blood = MachineSolve(user_blood_data, self._client)
-                self.root_blood_time = data['rootBlood']['blood_difference']
+                self.root_blood_time = parse_delta(data['rootBlood']['blood_difference'])
+        else:
+            self._is_summary = True
